\name{edge.lnL}
\alias{edge.lnL}

\title{ Likelihood Function for Edge Data }

\description{
     Compute the log-likelihood (to within an additive constant) of observing the data, under the multiple-edge model.
}

\usage{
     edge.lnL(params, edgecoord, observed, family="gaussian", neg=FALSE)
}

\arguments{
     \item{params}{The parameter values.}
     \item{edgecoord}{A dataframe of edges relocated to standard coordinates.}
     \item{observed}{A vector of observed response values, one for each focal point.}
     \item{family}{The error distribution of the observations.}
     \item{neg}{Whether to return the negative of the log-likelihood.}
}

\details{
    \code{params} must be given as a vector.  If it has names, they should be those given here.  If it has no names, the elements should be in this order.
    \describe{
        \item{e0}{The maximum effect (at distance 0)}
        \item{Dmax}{The maximum distance at which any effect is felt}
        \item{k}{The baseline value (at large distance)}
        \item{D0}{The distance out to which the maximum effect is felt.  If omitted, its value will be fixed to 0.}
     }

     The input \code{edgecoord} can be produced with \code{\link{relocate.edge.df}}.
     It is a dataframe with one row for each focal point--edge pair in the map.  The columns are:
     \describe{
          \item{map}{The name of the map containing the focal point--edge pair}
          \item{x0}{The x coordinate of the focal point (its y value is 0)}
          \item{y1}{The y coordinate of one end of the edge segment (its x value is 0)}
          \item{y2}{The y coordinate of the other end of the edge segment (its x value is 0)}
     }

     There are two options for \code{family}:
     \describe{
          \item{"gaussian"}{Gaussian errors}
          \item{"poisson"}{Poisson errors}
     }
}

\value{
     A floating point value that is proportional to the log-likelihood of the data under the multiple-edge model with the specified parameter values.
}

\note{
    Useful with \code{optim} or a general MCMC routine.
    See \code{vignette("edgefx")} for an example.
}

\references{
    Malcolm (1994) Ecology 75:2438--2445

    Ries etc.
}

\author{
    Goldberg, E.E. and Ries, L.

    Maintainer: Emma E. Goldberg <eeg@uic.edu>
}

\examples{
##---- Should be DIRECTLY executable !! ----
##-- ==>  Define data, use random,
##--	or do  help(data=index)  for the standard data sets.

## The function is currently defined as
function(params, edgecoord, observed, family="gaussian", neg=FALSE)
{
    if (!is.null(names(params)))
    {
        params <- c(params["e0"], params["Dmax"], params["k"], params["D0"])
    }
    # if the params vector doesn't have column names, assume it's in the right order
    if (length(params) == 3 || is.na(params[4]))
    {
        params[4] <- 0
    }

    if (params[2] < 0 || params[4] < 0 || params[4] > params[2])
    {
        lnL <- -Inf
    } else
    {
        xvals <- edgecoord[, 1]
        predicted <- as.vector(by(edgecoord[, 2:4], xvals, map.edge.effect, params[1], params[2], params[3], params[4]))[xvals]
        if (family == "gaussian")
        {
            lnL <- -sum((observed - predicted)^2)
            # could easily include weights:  diff / (2*var)
        } else if (family == "poisson")
        {
            if (any(predicted <= 0))
            {
                lnL <- -Inf
            } else
            {
                lnL <- sum(dpois(observed, predicted, log=T))
                # identical to:
                #    sum(-log(factorial(observed)) + observed*log(predicted) - predicted)
            }
        } else
        {
            stop(paste("ERROR: family", family, "not available; use gaussian or poisson."))
        }
    }
    if (neg)
    {
        return(-lnL)
    } else
    {
        return(lnL)
    }

}
}
